@@ -52,12 +52,12 @@ package com.openexchange.imap.cache;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.concurrent.Callable;
-import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Future;
 import java.util.concurrent.FutureTask;
 import javax.mail.MessagingException;
+import org.cliffc.high_scale_lib.NonBlockingHashMap;
 import com.openexchange.exception.OXException;
 import com.openexchange.imap.AccessedIMAPStore;
 import com.openexchange.mail.MailExceptionCode;
@@ -135,7 +135,7 @@ public final class ListLsubCache {
 
     private static final boolean DO_GETACL = true;
 
-    private static final ConcurrentMap<Key, ConcurrentMap<Integer, Future<ListLsubCollection>>> MAP = new ConcurrentHashMap<Key, ConcurrentMap<Integer, Future<ListLsubCollection>>>();
+    private static final ConcurrentMap<Key, ConcurrentMap<Integer, Future<ListLsubCollection>>> MAP = new NonBlockingHashMap<Key, ConcurrentMap<Integer, Future<ListLsubCollection>>>();
 
     /**
      * No instance
@@ -278,6 +278,12 @@ public final class ListLsubCache {
      */
     public static ListLsubEntry getCachedLSUBEntry(final String fullName, final int accountId, final IMAPFolder imapFolder, final Session session) throws OXException, MessagingException {
         final ListLsubCollection collection = getCollection(accountId, imapFolder, session);
+        if (isAccessible(collection)) {
+            final ListLsubEntry entry = collection.getLsub(fullName);
+            if (null != entry && (entry.canOpen() || entry.isNamespace())) {
+                return entry;
+            }
+        }
         synchronized (collection) {
             if (checkTimeStamp(imapFolder, collection)) {
                 final ListLsubEntry entry = collection.getLsub(fullName);
@@ -313,6 +319,12 @@ public final class ListLsubCache {
         try {
             final IMAPFolder imapFolder = (IMAPFolder) imapStore.getFolder(INBOX);
             final ListLsubCollection collection = getCollection(accountId, imapFolder, session);
+            if (isAccessible(collection)) {
+                final ListLsubEntry entry = collection.getList(fullName);
+                if (null != entry && (entry.canOpen() || entry.isNamespace())) {
+                    return entry;
+                }
+            }
             synchronized (collection) {
                 if (checkTimeStamp(imapFolder, collection)) {
                     final ListLsubEntry entry = collection.getList(fullName);
@@ -396,6 +408,12 @@ public final class ListLsubCache {
      */
     public static ListLsubEntry getCachedLISTEntry(final String fullName, final int accountId, final IMAPFolder imapFolder, final Session session) throws OXException, MessagingException {
         final ListLsubCollection collection = getCollection(accountId, imapFolder, session);
+        if (isAccessible(collection)) {
+            final ListLsubEntry entry = collection.getList(fullName);
+            if (null != entry && (entry.canOpen() || entry.isNamespace() || entry.hasChildren())) {
+                return entry;
+            }
+        }
         synchronized (collection) {
             if (checkTimeStamp(imapFolder, collection)) {
                 final ListLsubEntry entry = collection.getList(fullName);
@@ -428,12 +446,16 @@ public final class ListLsubCache {
         return false;
     }
 
+    private static boolean isAccessible(final ListLsubCollection collection) {
+        return !collection.isDeprecated() && ((System.currentTimeMillis() - collection.getStamp()) <= TIMEOUT);
+    }
+
     /**
      * Gets cached LIST/LSUB entry for specified full name.
      *
      * @param fullName The full name
      * @param accountId The account ID
-     * @param imapFolder The IMAP
+     * @param imapFolder The IMAP folder
      * @param session The session
      * @return The cached LIST/LSUB entry
      * @throws OXException If loading the entry fails
@@ -441,6 +463,14 @@ public final class ListLsubCache {
      */
     public static ListLsubEntry[] getCachedEntries(final String fullName, final int accountId, final IMAPFolder imapFolder, final Session session) throws OXException, MessagingException {
         final ListLsubCollection collection = getCollection(accountId, imapFolder, session);
+        if (isAccessible(collection)) {
+            ListLsubEntry listEntry = collection.getList(fullName);
+            if (null != listEntry && (listEntry.canOpen() || listEntry.isNamespace())) {
+                final ListLsubEntry lsubEntry = collection.getLsub(fullName);
+                final ListLsubEntry emptyEntryFor = ListLsubCollection.emptyEntryFor(fullName);
+                return new ListLsubEntry[] { listEntry, lsubEntry == null ? emptyEntryFor : lsubEntry };
+            }
+        }
         synchronized (collection) {
             if (checkTimeStamp(imapFolder, collection)) {
                 final ListLsubEntry listEntry = collection.getLsub(fullName);
@@ -465,6 +495,118 @@ public final class ListLsubCache {
         }
     }
 
+    /**
+     * Gets the LIST entry marked with "\Drafts" attribute.
+     * <p>
+     * Needs the <code>"SPECIAL-USE"</code> capability.
+     * 
+     * @param accountId The account identifier
+     * @param imapFolder The IMAP folder
+     * @param session The session
+     * @return The entry or <code>null</code>
+     * @throws OXException If loading the entry fails
+     * @throws MessagingException If a messaging error occurs
+     */
+    public static ListLsubEntry getDraftsEntry(final int accountId, final IMAPFolder imapFolder, final Session session) throws OXException, MessagingException {
+        final ListLsubCollection collection = getCollection(accountId, imapFolder, session);
+        if (isAccessible(collection)) {
+            return collection.getDraftsEntry();
+        }
+        synchronized (collection) {
+            if (checkTimeStamp(imapFolder, collection)) {
+                return collection.getDraftsEntry();
+            }
+            /*
+             * Return
+             */
+            return collection.getDraftsEntry();
+        }
+    }
+
+    /**
+     * Gets the LIST entry marked with "\Junk" attribute.
+     * <p>
+     * Needs the <code>"SPECIAL-USE"</code> capability.
+     * 
+     * @param accountId The account identifier
+     * @param imapFolder The IMAP folder
+     * @param session The session
+     * @return The entry or <code>null</code>
+     * @throws OXException If loading the entry fails
+     * @throws MessagingException If a messaging error occurs
+     */
+    public static ListLsubEntry getJunkEntry(final int accountId, final IMAPFolder imapFolder, final Session session) throws OXException, MessagingException {
+        final ListLsubCollection collection = getCollection(accountId, imapFolder, session);
+        if (isAccessible(collection)) {
+            return collection.getJunkEntry();
+        }
+        synchronized (collection) {
+            if (checkTimeStamp(imapFolder, collection)) {
+                return collection.getJunkEntry();
+            }
+            /*
+             * Return
+             */
+            return collection.getJunkEntry();
+        }
+    }
+
+    /**
+     * Gets the LIST entry marked with "\Sent" attribute.
+     * <p>
+     * Needs the <code>"SPECIAL-USE"</code> capability.
+     * 
+     * @param accountId The account identifier
+     * @param imapFolder The IMAP folder
+     * @param session The session
+     * @return The entry or <code>null</code>
+     * @throws OXException If loading the entry fails
+     * @throws MessagingException If a messaging error occurs
+     */
+    public static ListLsubEntry getSentEntry(final int accountId, final IMAPFolder imapFolder, final Session session) throws OXException, MessagingException {
+        final ListLsubCollection collection = getCollection(accountId, imapFolder, session);
+        if (isAccessible(collection)) {
+            return collection.getSentEntry();
+        }
+        synchronized (collection) {
+            if (checkTimeStamp(imapFolder, collection)) {
+                return collection.getSentEntry();
+            }
+            /*
+             * Return
+             */
+            return collection.getSentEntry();
+        }
+    }
+
+    /**
+     * Gets the LIST entry marked with "\Trash" attribute.
+     * <p>
+     * Needs the <code>"SPECIAL-USE"</code> capability.
+     * 
+     * @param accountId The account identifier
+     * @param imapFolder The IMAP folder
+     * @param session The session
+     * @return The entry or <code>null</code>
+     * @throws OXException If loading the entry fails
+     * @throws MessagingException If a messaging error occurs
+     */
+    public static ListLsubEntry getTrashEntry(final int accountId, final IMAPFolder imapFolder, final Session session) throws OXException, MessagingException {
+        final ListLsubCollection collection = getCollection(accountId, imapFolder, session);
+        if (isAccessible(collection)) {
+            return collection.getTrashEntry();
+        }
+        synchronized (collection) {
+            if (checkTimeStamp(imapFolder, collection)) {
+                return collection.getTrashEntry();
+            }
+            /*
+             * Return
+             */
+            return collection.getTrashEntry();
+        }
+    }
+
     private static ListLsubCollection getCollection(final int accountId, final IMAPFolder imapFolder, final Session session) throws OXException, MessagingException {
         /*
          * Initialize appropriate cache entry
@@ -472,7 +614,7 @@ public final class ListLsubCache {
         final Key key = keyFor(session);
         ConcurrentMap<Integer, Future<ListLsubCollection>> map = MAP.get(key);
         if (null == map) {
-            final ConcurrentMap<Integer, Future<ListLsubCollection>> newmap = new ConcurrentHashMap<Integer, Future<ListLsubCollection>>();
+            final ConcurrentMap<Integer, Future<ListLsubCollection>> newmap = new NonBlockingHashMap<Integer, Future<ListLsubCollection>>();
             map = MAP.putIfAbsent(key, newmap);
             if (null == map) {
                 map = newmap;
@@ -574,6 +716,9 @@ public final class ListLsubCache {
      */
     public static boolean hasAnySubscribedSubfolder(final String fullName, final int accountId, final IMAPFolder imapFolder, final Session session) throws OXException, MessagingException {
         final ListLsubCollection collection = getCollection(accountId, imapFolder, session);
+        if (isAccessible(collection)) {
+            return collection.hasAnySubscribedSubfolder(fullName);
+        }
         synchronized (collection) {
             checkTimeStamp(imapFolder, collection);
             return collection.hasAnySubscribedSubfolder(fullName);
