@@ -54,10 +54,11 @@ import java.net.InetSocketAddress;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.net.UnknownHostException;
+import java.util.Collections;
 import java.util.Map;
-import java.util.concurrent.ConcurrentHashMap;
 import javax.mail.MessagingException;
 import javax.mail.internet.IDNA;
+import org.cliffc.high_scale_lib.NonBlockingHashMap;
 import com.openexchange.exception.OXException;
 import com.openexchange.imap.IMAPCapabilities;
 import com.openexchange.imap.IMAPException;
@@ -97,6 +98,8 @@ public final class IMAPConfig extends MailConfig {
 
     private boolean secure;
 
+    private Boolean acl;
+
     private IIMAPProperties mailProperties;
 
     private InetAddress imapServerAddress;
@@ -115,7 +118,7 @@ public final class IMAPConfig extends MailConfig {
     public IMAPConfig(final int accountId) {
         super();
         this.accountId = accountId;
-        params = new ConcurrentHashMap<String, Object>(4);
+        params = new NonBlockingHashMap<String, Object>(4);
     }
 
     /**
@@ -180,7 +183,7 @@ public final class IMAPConfig extends MailConfig {
      * @return The (unmodifiable) map containing IMAP capabilities
      */
     public Map<String, String> asMap() {
-        return capabilities;
+        return capabilities == null ? Collections.<String, String> emptyMap() : capabilities;
     }
 
     /**
@@ -288,12 +291,25 @@ public final class IMAPConfig extends MailConfig {
         this.secure = secure;
     }
 
+    /**
+     * Sets the ACL flag.
+     * 
+     * @param acl <code>true</code> if ACL extension is supported; otherwise <code>false</code>
+     */
+    public void setAcl(boolean acl) {
+        this.acl = Boolean.valueOf(acl);
+    }
+
     /**
      * Checks if ACLs are supported
      *
      * @return <code>true</code> if ACLs are supported; otherwise <code>false</code>
      */
     public boolean isSupportsACLs() {
+        final Boolean acl = this.acl;
+        if (null != acl) {
+            return acl.booleanValue();
+        }
         final BoolCapVal supportsACLs = IMAPProperties.getInstance().getSupportsACLs();
         final Map<String, String> capabilities = this.capabilities;
         if (capabilities != null && BoolCapVal.AUTO.equals(supportsACLs)) {
