@@ -70,8 +70,8 @@ import com.openexchange.imap.config.IMAPProperties;
 import com.openexchange.imap.notify.IMAPNotifierRegistryService;
 import com.openexchange.imap.notify.internal.IMAPNotifierRegistry;
 import com.openexchange.imap.services.IMAPServiceRegistry;
-import com.openexchange.imap.thread.ThreadableCache;
-import com.openexchange.imap.thread.ThreadableLoginHandler;
+import com.openexchange.imap.threader.ThreadableCache;
+import com.openexchange.imap.threader.ThreadableLoginHandler;
 import com.openexchange.login.LoginHandlerService;
 import com.openexchange.mail.api.MailProvider;
 import com.openexchange.mailaccount.MailAccountStorageService;
